@@ -1,6 +1,6 @@
 ;;;; Common Lisp Object System for CLISP: Generic Functions
 ;;;; Bruno Haible 21.8.1993 - 2004
-;;;; Sam Steingold 1998 - 2004, 2007
+;;;; Sam Steingold 1998 - 2004, 2007 - 2010
 ;;;; German comments translated into English: Stefan Kain 2002-04-08
 
 (in-package "CLOS")
@@ -129,7 +129,10 @@
         ;; MOP p. 51 says that an error should be signalled in this case,
         ;; but ANSI CL says that CHANGE-CLASS is used to modify the GF.
         (change-class gf generic-function-class))
-      (warn-if-gf-already-called gf)
+      (when (need-gf-already-called-warning-p gf)
+        ;; maybe somehow check that this is not a simple file reloading?
+        (clos-warn 'simple-gf-already-called-warning
+          (TEXT "Redefining an already called generic function ~S") gf))
       (apply #'reinitialize-instance gf all-keys)
       gf)
     ;; First definition of a generic function.
@@ -216,8 +219,8 @@
   (multiple-value-bind (fast-function-factory-lambda method-initargs-forms signature)
       (analyze-method-description 'defmethod whole-form funname method-description)
     `(LET ()
-       (COMPILER::EVAL-WHEN-COMPILE
-         (COMPILER::C-DEFUN ',funname ,signature nil 'DEFMETHOD))
+       (SYS::EVAL-WHEN-COMPILE
+         (SYS::C-DEFUN ',funname ,signature nil 'DEFMETHOD))
        (WHEN (GET ',(SYS::GET-FUNNAME-SYMBOL funname) 'SYS::TRACED-DEFINITION)
          (SYS::UNTRACE1 ',funname))
        (DO-DEFMETHOD ',funname (FUNCTION ,fast-function-factory-lambda)
@@ -272,8 +275,8 @@
   (multiple-value-bind (fast-function-factory-lambda method-initargs-forms signature)
       (analyze-method-description 'defmethod whole-form funname method-description)
     (declare (ignore fast-function-factory-lambda method-initargs-forms))
-    `(COMPILER::EVAL-WHEN-COMPILE
-       (COMPILER::C-DEFUN ',funname ,signature nil 'DEFMETHOD))))
+    `(SYS::EVAL-WHEN-COMPILE
+       (SYS::C-DEFUN ',funname ,signature nil 'DEFMETHOD))))
 
 
 ;; ====================== DEFGENERIC and similar Macros ======================
@@ -509,10 +512,9 @@
 (defun analyze-defgeneric-lambdalist (caller whole-form funname lambdalist)
   (multiple-value-bind (reqvars optvars rest keyp keywords keyvars allowp)
       (sys::analyze-generic-function-lambdalist lambdalist
-        #'(lambda (detail errorstring &rest arguments)
-            (error-of-type 'ext:source-program-error
-              :form whole-form
-              :detail detail
+        #'(lambda (lalist detail errorstring &rest arguments)
+            (declare (ignore lalist)) ; use WHOLE-FORM instead
+            (sys::lambda-list-error whole-form detail
               (TEXT "~S ~S: invalid generic function lambda-list: ~A")
               caller funname (apply #'format nil errorstring arguments))))
     (declare (ignore keyvars))
@@ -533,8 +535,8 @@
            (method-combination-form `(,method-combination-lambda ,generic-function-class-var)))
       `(LET ()
          (DECLARE (SYS::IN-DEFUN ,funname))
-         (COMPILER::EVAL-WHEN-COMPILE
-           (COMPILER::C-DEFUN ',funname ',signature nil 'DEFGENERIC))
+         (SYS::EVAL-WHEN-COMPILE
+           (SYS::C-DEFUN ',funname ',signature nil 'DEFGENERIC))
          (WHEN (GET ',funname-symbol 'SYS::TRACED-DEFINITION)
            (SYS::UNTRACE1 ',funname))
          ;; NB: no (SYSTEM::REMOVE-OLD-DEFINITIONS ',funname)
@@ -573,7 +575,7 @@
              ;; argument list, where user-defined and default initargs can
              ;; override it. The MOP doesn't mandate this, but it allows users
              ;; to specify a default method-combination for a generic-function
-             ;; class; see clisp tracker item #[ 1415783 ].
+             ;; class; see https://sourceforge.net/p/clisp/bugs/316/
              ,@(if method-combination-p `(:METHOD-COMBINATION ,method-combination-form))
              ,@(if docstring-p `(:DOCUMENTATION ',docstring))
              ,@(if declspecs-p `(:DECLARATIONS ',declspecs))
