@@ -1,6 +1,6 @@
 ;;; Foreign function interface for CLISP
 ;;; Bruno Haible 19.2.1995
-;;; Sam Steingold 1998-2009
+;;; Sam Steingold 1998-2010
 
 #+UNICODE
 (progn
@@ -214,7 +214,7 @@
         (when name (setf (gethash name *c-type-table*) c-type))
         c-type)
       (if (stringp typespec)
-        (let ((c-type (parse-foreign-inttype typespec)))
+        (let ((c-type (parse-foreign-inttype typespec t)))
           (when name (setf (gethash name *c-type-table*) c-type))
           c-type)
         (error (TEXT "FFI type should be a symbol, not ~S")
@@ -782,7 +782,8 @@
     (let ((done (make-hash-table :test 'equal)))
       (maphash (lambda (type spec)
                  (declare (ignore type))
-                 (when (and (stringp spec) (not (gethash spec done)))
+                 (when (and (stringp spec) (not (gethash spec done))
+                            (eq 0 (parse-foreign-inttype spec nil)))
                    (setf (gethash spec done) spec)
                    (when *foreign-guard*
                      (format *coutput-stream* "# if HAVE_~A~%"
@@ -1171,7 +1172,7 @@
     (long (format nil "slong_to_I(~A)" argname))
     (ulong (format nil "ulong_to_I(~A)" argname))
     (single-float (format nil "c_float_to_FF((const ffloatjanus*)&~A)" argname))
-    (double-float (format nil "c_double_to_FF((const dfloatjanus*)&~A)" argname))
+    (double-float (format nil "c_double_to_DF((const dfloatjanus*)&~A)" argname))
     (c-pointer
      (let ((addr (format nil "(uintP)(*(void* const *) ~A)" argname)))
        (format nil "~A == 0 ? NIL : make_faddress(GLO(fp_zero),~A)"
@@ -1193,6 +1194,12 @@
     (ulong
      (format nil "if (ulong_p(~A)) *~A=I_to_ulong(~A); else error_ulong(~A)"
              lispobj retaddr lispobj lispobj))
+    (single-float
+     (format nil "if (!single_float_p(~A)) ~A=coerce_float(~A,S(single_float)); FF_to_c_float(~A,(ffloatjanus*)~A)"
+             lispobj lispobj lispobj lispobj retaddr))
+    (double-float
+     (format nil "if (!double_float_p(~A)) ~A=coerce_float(~A,S(double_float)); FF_to_c_float(~A,(ffloatjanus*)~A)"
+             lispobj lispobj lispobj lispobj retaddr))
     (t (format nil "convert_to_foreign(~A,~A,~A,&~A)"
                (object-to-c-value (pass-object rettype)) lispobj retaddr
                (if (flag-set-p flags ff-flag-malloc-free)
